<?php

namespace Config;

// Create a new instance of our RouteCollection class.
$routes = Services::routes();

// Load the system's routing file first, so that the app and ENVIRONMENT
// can override as needed.
if (is_file(SYSTEMPATH . 'Config/Routes.php')) {
    require SYSTEMPATH . 'Config/Routes.php';
}

/*
 * --------------------------------------------------------------------
 * Router Setup
 * --------------------------------------------------------------------
 */
$routes->setDefaultNamespace('App\Controllers');
$routes->setDefaultController('Login');
$routes->setDefaultMethod('index');
$routes->setTranslateURIDashes(false);
$routes->set404Override();
// The Auto Routing (Legacy) is very dangerous. It is easy to create vulnerable apps
// where controller filters or CSRF protection are bypassed.
// If you don't want to define all routes, please use the Auto Routing (Improved).
// Set `$autoRoutesImproved` to true in `app/Config/Feature.php` and set the following to true.
//$routes->setAutoRoute(false);

/*
 * --------------------------------------------------------------------
 * Route Definitions
 * --------------------------------------------------------------------
 */

// We get a performance increase by specifying the default
// route since we don't have to scan directories.
$routes->get('/', 'Login::index');
// $routes->add('welcome/(:any)', 'Welcome::index/$1/$2');

$routes->add('blog', function() {
    return "<h2>This is a blog</h2>";
});


// $routes->group('admin', static function ($routes) {
//     $routes->get('users', 'Admin\Users::index');
//     $routes->get('blog', 'Admin\Blog::index');
// });

//$routes->group('welcome', function ($routes) {
//    $routes->add('dashboard', 'Welcome::index');
//     $routes->get('test', 'Welcome::test');
//});

$routes->get('login', 'Login::auth');
$routes->post('login', 'Login::auth');
$routes->get('admin/dashboard', 'Admin::Dashboard');
$routes->get('admin/logout', 'Login::logout');

//update settings
$routes->get('systemsetting/system_settings', 'Systemsetting::system_settings');

//upload logo and Settings
$routes->post('systemsetting/system_settings/(:any)', 'Systemsetting::system_settings/$1');

//manage admin profile
$routes->get('admin/manage_profile', 'Admin::manage_profile');
$routes->post('admin/manage_profile/(:any)', 'Admin::manage_profile/$1');

// enquiry Navigation Menu GET Routes
$routes->get('admin/enquiry_category', 'Admin::enquiry_category');
$routes->get('admin/list_enquiry', 'Admin::list_enquiry');
$routes->get('admin/club', 'Admin::club');
$routes->get('admin/circular', 'Admin::circular');
$routes->get('admin/librarian', 'Admin::librarian');
$routes->get('admin/accountant', 'Admin::accountant');
$routes->get('admin/hostel', 'Admin::hostel');
$routes->get('admin/hrm', 'Admin::hrm');
$routes->get('admin/parent', 'Admin::parent');
$routes->get('admin/alumni', 'Admin::alumni');
$routes->get('admin/accountant/(:any)', 'Admin::accountant/$1');
$routes->get('admin/librarian/(:any)', 'Admin::librarian/$1');
$routes->get('admin/hostel/(:any)', 'Admin::hostel/$1');
$routes->get('admin/hrm/(:any)', 'Admin::hrm/$1');
$routes->get('admin/parent/(:any)', 'Admin::parent/$1');
$routes->get('admin/alumni/(:any)', 'Admin::alumni/$1');

// GET Routes For Popups
$routes->get('modal/popup/(:any)', 'Modal::popup/$1');


$routes->post('admin/enquiry_category/(:any)', 'Admin::enquiry_category/$1');

$routes->post('admin/manage_profile/(:any)', 'Admin::manage_profile/$1');

$routes->post('admin/list_enquiry/(:any)', 'Admin::list_enquiry/$1');

$routes->post('admin/club/(:any)', 'Admin::club/$1');
$routes->post('admin/circular/(:any)', 'Admin::circular/$1');
$routes->post('admin/librarian/(:any)', 'Admin::librarian/$1');
$routes->post('admin/accountant/(:any)', 'Admin::accountant/$1');
$routes->post('admin/hostel/(:any)', 'Admin::hostel/$1');
$routes->post('admin/hrm/(:any)', 'Admin::hrm/$1');
$routes->post('admin/parent/(:any)', 'Admin::parent/$1');
$routes->post('admin/alumni/(:any)', 'Admin::alumni/$1');







/*
 * --------------------------------------------------------------------
 * Additional Routing
 * --------------------------------------------------------------------
 *
 * There will often be times that you need additional routing and you
 * need it to be able to override any defaults in this file. Environment
 * based routes is one such time. require() additional route files here
 * to make that happen.
 *
 * You will have access to the $routes object within that file without
 * needing to reload it.
 */
if (is_file(APPPATH . 'Config/' . ENVIRONMENT . '/Routes.php')) {
    require APPPATH . 'Config/' . ENVIRONMENT . '/Routes.php';
}
